unit hiCallback;

interface

uses Windows,Kol,Share,Debug,Messages;

type
  THICallback = class(TDebug)
    private
    FEnabledCallbak:boolean;
    FTimer:PTimer;
    FDataMsg:TData;
    FDtNull:TData;
    FWndHandle:HWND;
    FWndCallback:Pointer;
    procedure AutoStart(Value:boolean);
    procedure Time(Value:integer);
    function WndHandleCallback(HWnd: THandle; Msg: Cardinal;
                 WParam, LParam: Integer): Cardinal; Stdcall;
  public
     _event_onKeyUp:THI_Event;
     _event_onKeyDown:THI_Event;
     _event_onShow:THI_Event;
     _event_onResize:THI_Event;
     _event_onMouseDown:THI_Event;
     _event_onMouseMove:THI_Event;
     _event_onMouseUp:THI_Event;
     _event_onMouseWheel:THI_Event;
     _event_onPaint:THI_Event;
     _event_onDblClick:THI_Event;
     _event_onSetFocus:THI_Event;
     _event_onKillFocus:THI_Event;
     _event_onClose:THI_Event;

     _data_Handle:THI_Event;
     _prop_OutMtMsg:boolean;
     _prop_OutCaption:boolean;

     constructor Create;
     destructor Destroy; override;
     procedure _work_doCreateHook(var _Data:TData; Index:word);
     procedure _work_doStopHook(var _Data:TData; Index:word);
     procedure MsgToMTdata(var dtmt_msg: TDAta;
                           Msg: Cardinal; WParam, LParam: Integer);
     procedure _ToEvent(Msg: Cardinal);
     procedure OnTimer(Obj:PObj);
     property WndCallback: Pointer read FWndCallback;
     property _prop_AutoTimer: boolean write AutoStart;
     property _prop_Time: integer write Time;
  end;

implementation

procedure THICallback.AutoStart;
begin
   FTimer.Enabled := Value;
end;

procedure THICallback.Time;
begin
  FTimer.Interval := Value;
end;

procedure THICallback.OnTimer;
var dt:TData;
begin
  FTimer.Enabled := false;
  FTimer.Free;
  _work_doCreateHook(dt,0);
end;

constructor THICallback.Create;
begin
  inherited Create;
  FTimer := NewTimer(1000);
  FTimer.Enabled := false;
  FTimer.OnTimer := onTimer;
  FDtNull.Data_type := data_null;
  FEnabledCallbak := true;
  FWndHandle := 0;
  FWndCallback := VirtualAlloc(nil, 12, MEM_RESERVE or MEM_COMMIT,
    PAGE_EXECUTE_READWRITE);
end;

destructor THICallback.Destroy;
begin
  FEnabledCallbak := false;
  VirtualFree(FWndCallback, 0, MEM_RELEASE);
  FreeData(@FDataMsg);
  FreeData(@FDtNull);
  inherited Destroy;
end;

procedure THICallback.MsgToMTdata(var dtmt_msg: TDAta;
            Msg: Cardinal; WParam, LParam: Integer);
var mtmsg,mtwp,mtlp,
    mtwplo,mtwphi,
    mtlplo,mtlphi:TData;
    s:PData;
begin
  dtNull(dtmt_msg);
  dtNull(mtwp); dtNull(mtlp);
  dtNull(mtwplo);dtNull(mtwphi);
  dtNull(mtlplo);dtNull(mtlphi);
  dtInteger(mtwp,WParam);
  dtInteger(mtlp,LParam);
  dtInteger(mtwplo,LoWord(WParam));
  dtInteger(mtwphi,HiWord(WParam));
  dtInteger(mtlplo,LoWord(LParam));
  dtInteger(mtlphi,HiWord(LParam));
  AddMTData(@FDataMsg,@mtwp,s);
  AddMTData(@FDataMsg,@mtlp,s);
  AddMTData(@FDataMsg,@mtwplo,s);
  AddMTData(@FDataMsg,@mtwphi,s);
  AddMTData(@FDataMsg,@mtlplo,s);
  AddMTData(@FDataMsg,@mtlphi,s);
end;

function THICallback.WndHandleCallback(HWnd: THandle; Msg: Cardinal;
             WParam, LParam: Integer): Cardinal; Stdcall;
begin
  case FEnabledCallbak of
    true: begin
      case _prop_OutMtMsg of
        true:  begin
                  case Msg of
                    WM_LBUTTONDOWN:   begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onMouseDown, FDataMsg); end;

                    WM_LBUTTONUP:     begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onMouseUp, FDataMsg); end;

                    WM_LBUTTONDBLCLK: begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onDblClick, FDataMsg); end;

                    WM_MOUSEMOVE:     begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onMouseMove, FDataMsg); end;

                    WM_MOUSEWHEEL:    begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onMouseWheel, FDataMsg); end;

                    WM_KEYUP:         begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onKeyUp, FDataMsg); end;

                    WM_KEYDOWN:       begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onKeyDown, FDataMsg); end;

                    WM_PAINT:         begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onPaint, FDataMsg); end;

                    WM_SIZE:          begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onResize, FDataMsg); end;

                    WM_SETFOCUS:      begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onSetFocus, FDataMsg); end;

                    WM_KILLFOCUS:     begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onKillFocus, FDataMsg); end;

                    WM_SHOWWINDOW:    begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onShow, FDataMsg); end;

                    WM_ClOSE:         begin MsgToMTdata(FDataMsg,Msg,WParam,LParam);
                                      _hi_OnEvent(_event_onClose, FDataMsg); end;
                  end;
               end;
        false: _ToEvent(Msg);
      end;
    end;
  end;

  Result := CallWindowProc(Pointer(GetWindowLong(HWnd, GWL_USERDATA)),
                Hwnd, Msg, WParam, LParam);
end;

procedure THICallback._work_doCreateHook;
begin
  FEnabledCallbak := true;
  if FWndHandle = 0 then begin
     FWndHandle := ReadInteger(_Data,_data_Handle);
    asm
      mov  EAX, Self
      mov  ECX, [EAX].THICallback.FWndCallback
      mov  word  ptr [ECX+0], $6858
      mov  dword ptr [ECX+2], EAX
      mov  word  ptr [ECX+6], $E950
      mov  EAX, OFFSET(THICallback.WndHandleCallback)
      sub  EAX, ECX
      sub  EAX, 12
      mov  dword ptr [ECX+8], EAX
    end;

    SetWindowLong(FWndHandle, GWL_USERDATA,
      SetWindowLong(FWndHandle, GWL_WNDPROC, LongInt(FWndCallback)));
  end else exit;
end;

procedure THICallback._work_doStopHook;
begin
  FEnabledCallbak := false;
end;

procedure THICallback._ToEvent;
begin
if _prop_OutCaption then begin
  case Msg of
    WM_LBUTTONDOWN:    _hi_OnEvent(_event_onMouseDown, 'onMouseDown');
    WM_LBUTTONUP:      _hi_OnEvent(_event_onMouseUp, 'onMouseUp');
    WM_LBUTTONDBLCLK:  _hi_OnEvent(_event_onDblClick, 'onDblClick');
    WM_MOUSEMOVE:      _hi_OnEvent(_event_onMouseMove, 'onMouseMove');
    WM_MOUSEWHEEL:     _hi_OnEvent(_event_onMouseWheel, 'onMouseWheel');
    WM_KEYUP:          _hi_OnEvent(_event_onKeyUp, 'onKeyUp');
    WM_KEYDOWN:        _hi_OnEvent(_event_onKeyDown, 'onKeyDown');
    WM_PAINT:          _hi_OnEvent(_event_onPaint, 'onPaint');
    WM_SIZE:           _hi_OnEvent(_event_onResize, 'onResize');
    WM_SETFOCUS:       _hi_OnEvent(_event_onSetFocus, 'onSetFocus');
    WM_KILLFOCUS:      _hi_OnEvent(_event_onKillFocus, 'onKillFocus');
    WM_SHOWWINDOW:     _hi_OnEvent(_event_onShow, 'onShow');
    WM_CLOSE:          _hi_OnEvent(_event_onClose,'onClose');
  end;
    end else begin
  case Msg of
    WM_LBUTTONDOWN:    _hi_OnEvent(_event_onMouseDown, FDtNull);
    WM_LBUTTONUP:      _hi_OnEvent(_event_onMouseUp, FDtNull);
    WM_LBUTTONDBLCLK:  _hi_OnEvent(_event_onDblClick, FDtNull);
    WM_MOUSEMOVE:      _hi_OnEvent(_event_onMouseMove, FDtNull);
    WM_MOUSEWHEEL:     _hi_OnEvent(_event_onMouseWheel, FDtNull);
    WM_KEYUP:          _hi_OnEvent(_event_onKeyUp, FDtNull);
    WM_KEYDOWN:        _hi_OnEvent(_event_onKeyDown, FDtNull);
    WM_PAINT:          _hi_OnEvent(_event_onPaint, FDtNull);
    WM_SIZE:           _hi_OnEvent(_event_onResize, FDtNull);
    WM_SETFOCUS:       _hi_OnEvent(_event_onSetFocus, FDtNull);
    WM_KILLFOCUS:      _hi_OnEvent(_event_onKillFocus, FDtNull);
    WM_SHOWWINDOW:     _hi_OnEvent(_event_onShow, FDtNull);
    WM_CLOSE:          _hi_OnEvent(_event_onClose, FDtNull);
  end;
end;
end;

initialization

finalization

end.
